#include <gtk/gtk.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <linux/videodev.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "bttv.h"
#include "event.h"
#include "save_video.h"

// image area for display
static GdkImlibImage* im = NULL;

// pixmap area for backingstore
static GdkPixmap* pixmap = NULL;

gint buf_px = 32768, buf_py = 32768;
gint file_num=0;

// backingstore Pixmap area by change setting
gint configure_event(GtkWidget* widget, GdkEventConfigure* event)
{
  gint w, h;

  // make display image of new size
  w = widget->allocation.width;
  h = widget->allocation.height;
  gdk_imlib_render(im, w, h);

  // make new Pixmap from display image of new size
  if (pixmap) gdk_pixmap_unref(pixmap);
  pixmap = gdk_imlib_move_image(im);

  return TRUE;
}

// renewal of screen from Pixmap for backingstore
gint expose_event(GtkWidget* widget, GdkEventExpose* event)
{
  if (pixmap) gdk_draw_pixmap(widget->window, widget->style->
			      fg_gc[GTK_WIDGET_STATE (widget)], 
			      pixmap,
			      event->area.x, event->area.y,
			      event->area.x, event->area.y,
			      event->area.width, event->area.height);

  return TRUE;
}

// renewal of screen
gint update_screen(GtkWidget* widget)
{
  GdkEventExpose e;

  e.type = GDK_EXPOSE;
  e.window = widget->window;
  e.area.x = 0;
  e.area.y = 0;
  e.area.width = widget->allocation.width;
  e.area.height = widget->allocation.height;
  e.send_event = 0;
  e.count = 0;

  // renewal screen and display
  if (e.window != NULL)
    {
      im = get_next_frame(im);

      configure_event(widget, NULL);
      expose_event(widget, &e);

      save_video();
      //      printf("a");
    }

  // return TRUE for called update continue timeout function
  return TRUE;
}


//̤򥯥åβ¸
gint capture(GtkObject* widget){
// video ... file discripta
  int fd;
  FILE* fp;
  char filename[20];

  // pointer for capture data
  unsigned char* buf;

  // video capture attribute
  struct video_capability vcap;
  struct video_channel vch;
  struct video_mbuf vbuf;
  struct video_mmap vmm;

  int i;

  inline void
    swap_rb(int i, int j, unsigned char* buf)
    {
      unsigned char tmp;
      
      tmp = buf[i+2];
      buf[i+2] = buf[i];
      buf[i] = tmp;
    }


  // video device open
  if ((fd = open(V4L_DEVICE, O_RDWR)) <= 0){
      perror("Video Device Open");
      exit(-1);
    }

  // Video device infomation get
  if (ioctl(fd, VIDIOCGCAP, &vcap) < 0){
      perror("VIDIOCGCAP");
      exit(-1);
    }

  if ((V4L_WIDTH > vcap.maxwidth) || (V4L_HEIGHT > vcap.maxheight)  ||
      (V4L_WIDTH < vcap.minwidth) || (V4L_HEIGHT < vcap.minheight)){
      fprintf(stderr, "Illegal Size Definition");
      exit(-1);
    }

  // mmap information get
  if (ioctl(fd, VIDIOCGMBUF, &vbuf) < 0){
      perror("VIDIOCGMBUF");
      exit(-1);
    }

  // suppose double buffer
  if (vbuf.frames < 2){
       perror("NO DOUBLE BUFFER");
       exit(-1);
    }

  // mmap
  if ((buf = (unsigned char*)mmap(0, vbuf.size, PROT_READ | PROT_WRITE, 
				  MAP_SHARED, fd, 0)) == MAP_FAILED){
      perror("MMAP");
      exit(-1);
    }

  // setting input signal
  vch.channel = 0;
  vch.norm = VIDEO_MODE_NTSC;
  if(ioctl(fd, VIDIOCSCHAN, &vch) < 0){
      perror("VIDIOCSCHAN");
      exit(-1);
    }

  // capturedate detail setting
  vmm.width = V4L_WIDTH;
  vmm.height = V4L_HEIGHT;
  vmm.format = VIDEO_PALETTE_RGB24;

  // start capture
  vmm.frame = 0;
  if (ioctl(fd, VIDIOCMCAPTURE, &vmm) < 0){
      perror("VIDIOCMCAPTURE(0)");
      exit(-1);
    }

  // waiting for capture end
  if(ioctl(fd, VIDIOCSYNC, &vmm.frame) < 0){
      perror("VIDIOCSYNC");
      exit(-1);
    }

  // BGR -> RGB
  buf += vbuf.offsets[vmm.frame];
  for (i = 0; i < V4L_WIDTH * V4L_HEIGHT * 3; i += 3){
    swap_rb(i, i+2, buf);
  }
  
  // ppm output
  sprintf( filename, "image%02d.ppm", file_num);
  if( (fp = fopen( filename, "wb+") ) < 0){
    printf("file open error.\n");
  }
  fprintf(fp,"P6 %d %d 255\n", V4L_WIDTH, V4L_HEIGHT); 
  //fflush(stdout);
  fwrite(buf, sizeof(char), V4L_WIDTH * V4L_HEIGHT * 3, fp);
  printf("save image%02d.ppm \n", file_num);
  file_num += 1;

  fclose(fp); 
  close(fd);
  return 0;
}


gint coordinate(GtkWidget *widget, GdkEventButton *ev){
  printf("(x y) = (%.f %.f)\n", ev->x, ev->y);
  return 0;
}

gint control(GtkWidget *widget, GdkEventButton *ev){
  FILE* fd;
  gdouble x, y, dx, dy, lsb_x, lsb_y;
  gint px, py, p[8], i;
  gchar data[20];

  //pixel (x,y)
  x = ev->x;
  y = ev->y;
  printf("(x y) = (%.f %.f)\n", x, y);

  //[division] ratina center and (x,y)
  dx = x - 159;
  dy = y - 119;
  
  //pixel -> angle measure -> LSB
  lsb_x = dx * 10/63 / 0.1125;
  lsb_y = dy * 10/63 / 0.1125;
  //printf("LSB(x y) = (%.2f %.2f)\n", lsb_x, lsb_y);

  //parameter [pan tilt]
  px = lsb_x + buf_px;  
  py = -lsb_y + buf_py; //lsb_y"-"դƤΤϥȤοϲ
  printf("param(x y) = (%x %x)\n", px, py);

  //Pan and Tilt limit process
  if( px >= 33657 || px <= 31879){
    printf("**Pan Limit! Can't move.**\n");
    return 0;
  }else if( py >= 33035 || py <= 32501){
    printf("**Tilt Limit! Can't move.**\n");
    return 0;
  }

  //make signal 
  p[0] = (px / (16*16*16)) % 16;
  p[1] = (px / (16*16)) % 16;
  p[2] = (px / 16) % 16;
  p[3] = px % 16;
  p[4] = (py / (16*16*16)) % 16;
  p[5] = (py / (16*16)) % 16;
  p[6] = (py / 16) % 16;
  p[7] = py % 16;

  buf_px = px;
  buf_py = py;
  
  for(i=0;i<8;i++){
    if(p[i] > 9){
      p[i] += 55;
    }else if(p[i] <= 9 && p[i] >= 0){
      p[i] += 48;     
    }
  }
  
  sprintf(data,"\xFF\x30\x30%c\x62%c%c%c%c%c%c%c%c\xEF",
	  0x00, p[0], p[1], p[2], p[3], p[4], p[5], p[6], p[7]);
  
  //send signal
  fd = fopen("/dev/ttyS0","rb+");
  if(fd < 0) printf("file open error\n");
  fwrite(data, 1, 15, fd);
  fclose(fd);

  return 0;
}
